# =============================================================================
# Copyright 2025 AutoFiC Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================

import subprocess
from pydantic import BaseModel  

class SemgrepResult(BaseModel):
    stdout: str
    stderr: str
    returncode: int

class SemgrepRunner(BaseModel):
    repo_path: str
    rule: str

    def run_semgrep(self) -> SemgrepResult: 
        cmd = [
            "semgrep",
            "--config", self.rule,
            "--json",
            "--include", "*.js", 
            "--include", "*.jsx", 
            "--include", "*.mjs",
            self.repo_path
        ]
    
        try:
            completed = subprocess.run(cmd, capture_output=True, text=True, encoding='utf-8', check=True)
            return SemgrepResult(stdout=completed.stdout, stderr=completed.stderr, returncode=completed.returncode)
        except subprocess.CalledProcessError as err:
            return SemgrepResult(stdout=err.stdout, stderr=err.stderr, returncode=err.returncode)